using System;
using System.Collections.Generic;
using System.Text;
using System.Web.UI;

namespace Singular.Web.UI.Controls
{
	public class Script : Control, IComparable, IComparable<Script>, IComparable<string>
	{
		public const string DefaultContentType = "text/javascript";

		#region Private Variables

		string _key;
		string _content;
		bool _external;
		string _contentType;
		bool _defer = false;
		bool _compress = false;
		bool _minify = false;

		#endregion

		#region Constructors

		/// <summary>
		/// 
		/// </summary>
		public Script()
			: this( string.Empty, false )
		{
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="content"></param>
		/// <param name="external"></param>
		public Script( string content, bool external )
			: this( null, content, DefaultContentType, external )
		{
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="content"></param>
		/// <param name="contentType"></param>
		/// <param name="external"></param>
		public Script( string content, string contentType, bool external )
			: this( null, content, contentType, external )
		{
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="key"></param>
		/// <param name="content"></param>
		/// <param name="contentType"></param>
		/// <param name="external"></param>
		public Script( string key, string content, string contentType, bool external )
		{
			_key = key;
			_content = content;
			_contentType = contentType;
			_external = external;

			EnableViewState = false;
		}

		#endregion

		#region Properties

		/// <summary>
		/// The key that the script should be known as
		/// </summary>
		public string Key
		{
			get { return _key ?? _content; }
			set { _key = value; }
		}

		/// <summary>
		/// The script's external src or actual content
		/// </summary>
		public string Content
		{
			get { return _content; }
			set { _content = value; }
		}

		/// <summary>
		/// Sets the Content to the given value, and ensures the control is 
		/// marked as having an external source
		/// </summary>
		public string Src
		{
			get { return _external ? _content : null; }
			set
			{
				_content = value;
				_external = true;
			}
		}

		/// <summary>
		/// Whether the script is external or not
		/// </summary>
		public bool External
		{
			get { return _external; }
			set { _external = value; }
		}

		/// <summary>
		/// The content-type of the script. Default of text/javascript
		/// </summary>
		public string ContentType
		{
			get { return _contentType; }
			set { _contentType = value; }
		}

		/// <summary>
		/// 
		/// </summary>
		public bool Defer
		{
			get { return _defer; }
			set { _defer = value; }
		}

		/// <summary>
		/// Placeholder for dynamic http compression
		/// </summary>
		public bool Compress
		{
			get { return _compress; }
			set { _compress = value; ; }
		}

		/// <summary>
		/// Placeholder for dynamic script minification
		/// </summary>
		public bool Minify
		{
			get { return _minify; }
			set { _minify = value; ; }
		}

		#endregion

		#region Methods

		#region ToString

		/// <summary>
		/// 
		/// </summary>
		/// <returns></returns>
		public override string ToString()
		{
			return _content;
		}

		#endregion

		#region IComparable Members

		public int CompareTo( object obj )
		{
			if( obj is Script )
			{
				return CompareTo( (Script)obj );
			}
			else if( obj is string )
			{
				return CompareTo( (string)obj );
			}

			throw new ArgumentException( "Argument is of the wrong type." );
		}

		#endregion

		#region IComparable<Script> Members

		public int CompareTo( Script other )
		{
			return Key.CompareTo( other.Key );
		}

		#endregion

		#region IComparable<string> Members

		public int CompareTo( string other )
		{
			return Key.CompareTo( other );
		}

		#endregion

		#region Render

		/// <summary>
		/// 
		/// </summary>
		/// <param name="writer"></param>
		protected override void Render( HtmlTextWriter writer )
		{
			if( External ) renderScriptExternal( writer );
			else renderScriptContent( writer );
		}

		#endregion

		#region renderScriptContent

		/// <summary>
		/// 
		/// </summary>
		/// <param name="writer"></param>
		void renderScriptContent( HtmlTextWriter writer )
		{
			writer.WriteBeginTag( "script" );
			writer.WriteAttribute( "type", ContentType );
			writer.Write( HtmlTextWriter.TagRightChar );
			writer.Write( Content );
			writer.WriteEndTag( "script" );
		}

		#endregion

		#region renderScriptExternal

		/// <summary>
		/// 
		/// </summary>
		/// <param name="writer"></param>
		void renderScriptExternal( HtmlTextWriter writer )
		{
			writer.WriteBeginTag( "script" );
			writer.WriteAttribute( "type", ContentType );
			writer.WriteAttribute( "src", ResolveUrl( Content ) );
			if( _defer )
			{
				writer.WriteAttribute( "defer", "defer" );
			}
			writer.Write( HtmlTextWriter.TagRightChar );
			writer.WriteEndTag( "script" );
		}

		#endregion

		#endregion

	}
}
